@@ -64,8 +64,3 @@ class PeakGrid(GridModel):
             ) * self.time_step_len
         else:
             return power * self.base_price * self.time_step_len
-
-    def get_info(self):
-        """Get info about price threshold."""
-
-        return {"price_threshold": self.peak_threshold}
