@@ -10,12 +10,17 @@ def register_all() -> None:
         entry_point="bauwerk.envs.solar_battery_house:SolarBatteryHouseEnv",
     )
 
+    gym.envs.registration.register(
+        id="bauwerk/House-v0",
+        entry_point="bauwerk.envs:HouseEnv",
+    )
+
     gym.envs.registration.register(
         id="bauwerk/BuildDistA-v0",
-        entry_point="bauwerk.envs.distributions:sample_build_dist_a",
+        entry_point="bauwerk.envs.distributions:BuildDistAEnv",
     )
 
     gym.envs.registration.register(
         id="bauwerk/BuildDistB-v0",
-        entry_point="bauwerk.envs.distributions:sample_build_dist_b",
+        entry_point="bauwerk.envs.distributions:BuildDistBEnv",
     )
