@@ -48,8 +48,59 @@ class EnvPlotter:
         plot_actions: bool = True,
         rescale_action: bool = True,
         plot_optimal_acts: bool = True,
+        interactive_mode: bool = False,
     ) -> None:
-        """Plotting class for Bauwerk environments."""
+        """Plotting class for Bauwerk environments.
+
+        Additional information about the plot:
+
+        * The blue dashed line in the top plot
+            represents the power threshold above which
+            energy costs more than below.
+        * The net load in the top plot is the load
+            that has be covered using energy from the
+            grid. It's the negative residential load
+            plus battery (dis)charge plus PV generation.
+        * In the third plot from the top, a red line is
+            shown if the action taken by the agent is
+            not feasible, and has to be corrected
+            inside the environment.
+        * In the bottom plot, the dotted line is only
+            shown if a penalty is actually applied
+            (this may not be case either because the
+            actions are all feasible or because the
+            penalty is not activated).
+
+        Args:
+            initial_obs (dict): initial observations
+            env (bauwerk.HouseEnv): environment to plot trajectories from.
+            visible_h (int, optional): hours of trajectory visible in plot.
+                Defaults to 24.
+            fig_height (int, optional): height of figure in px. Defaults to 600.
+            debug_mode (bool, optional): whether to run in debug mode. Defaults to
+                False.
+            include_house_figure (bool, optional): whether to include figure of house.
+                Mostly useful in the context of interactive plotting as
+                part of Bauwerk game.
+                Defaults to False.
+            alternative_plotting (bool, optional): Two variations of plotting are
+                available. The second (alternative) variation is recommended and the
+                default. Defaults to True.
+            include_clock_in_house_figure (bool, optional): whether to include a
+                clock in the house figure (if this one is active). Defaults to True.
+            score_currency (str, optional): currency shown in score. Defaults to "€".
+            background (Optional[str], optional): background color.
+                Defaults to "white".
+            plot_grid_threshold (bool, optional): whether to plot peak demand pricing
+                threshold of grid. Defaults to True.
+            plot_actions (bool, optional): whether to plot actions. Defaults to True.
+            rescale_action (bool, optional): whether to rescale actions.
+                Defaults to True.
+            plot_optimal_acts (bool, optional): whether to include a
+                plot of the optimal actions. Defaults to True.
+            interactive_mode (bool, optional): whether Figures should be redrawn for
+                interactive usage, like in Bauwerk game. Defaults to False.
+        """
 
         self.reward_label = "Reward (payment)"
         self.score_currency = score_currency
@@ -66,6 +117,7 @@ class EnvPlotter:
         self.plot_actions = plot_actions
         self.rescale_action = rescale_action
         self.plot_optimal_acts = plot_optimal_acts
+        self.interactive_mode = interactive_mode
 
         if self.plot_optimal_acts:
             self.optimal_acts = bauwerk.solve(env)[0]
@@ -75,7 +127,7 @@ class EnvPlotter:
 
     def _add_obs(self, obs):
         for key in self.obs_values.keys():
-            self.obs_values[key].append(obs[key])
+            self.obs_values[key].append(float(obs[key]))
 
     def _set_up_figure(self) -> None:
 
@@ -401,6 +453,8 @@ class EnvPlotter:
             self.time_day_text.set_text(f"Day {int(days)}")
 
     def update_figure(self):
+        """Update the figure to current data."""
+
         if not self.alternative_plotting:
             for i, obs_part in enumerate(self.obs_values.values()):
                 # setting new data
@@ -463,8 +517,10 @@ class EnvPlotter:
         if self.include_house_figure:
             self._update_house_figure()
 
-        self.fig.canvas.draw()
-        self.fig.canvas.flush_events()
+        # https://matplotlib.org/stable/users/explain/interactive_guide.html
+        if self.interactive_mode:
+            self.fig.canvas.draw_idle()
+            self.fig.canvas.flush_events()
 
     def step(self, action, observation, reward):
         self._add_obs({**observation, self.reward_label: reward, "action": action})
@@ -474,7 +530,14 @@ class EnvPlotter:
 
         self.update_figure()
 
-    def add_step_data(self, step_return, action):
+    def add_step_data(self, step_return: tuple, action: np.array) -> None:
+        """Add data from stepping in the environment.
+
+        Args:
+            step_return (tuple): value returned by stepping
+                in environment (i.e. by ``env.step(action)``).
+            action (np.array): action that was given to ``env.step()``.
+        """
         observation = step_return[0]
         reward = step_return[1]
         info = step_return[-1]
@@ -498,7 +561,12 @@ class EnvPlotter:
         self.reward += reward
         self.current_step += 1
 
-    def reset(self, obs):
+    def reset(self, obs: dict) -> None:
+        """Reset figure with observation returned by ``env.reset()``.
+
+        Args:
+            obs (dict): initial observations returned by ``env.reset()``.
+        """
         obs = {
             **obs,
             self.reward_label: np.array([0], dtype=float),
@@ -512,7 +580,7 @@ class EnvPlotter:
             "info_cost": np.array([0], dtype=float),
         }
         self.obs_values = {
-            key: [np.array([0], dtype=float)] * (self.visible_steps + 1)
+            key: [0] * (self.visible_steps + 1)
             for key in obs.keys()
             if key not in ["time_step", "time_of_day"]
         }
